package com.ailanyin.admin.service.impl;

import com.ailanyin.admin.service.SysRoleService;
import com.ailanyin.admin.service.SysUserService;
import com.ailanyin.common.annotation.DataScope;
import com.ailanyin.common.exception.Asserts;
import com.ailanyin.common.utils.EmptyUtil;
import com.ailanyin.common.utils.SpringUtil;
import com.ailanyin.common.utils.StringUtil;
import com.ailanyin.mapper.*;
import com.ailanyin.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author ailanyin
 * @version 1.0
 * @since 2021/10/15 0015 下午 13:03
 */
@Service
@EnableAspectJAutoProxy(proxyTargetClass = true, exposeProxy = true)
public class SysRoleServiceImpl implements SysRoleService {

    @Autowired
    private SysUserService sysUserService;

    @Resource
    private SysMenuMapper menuMapper;
    @Resource
    private SysRoleMapper roleMapper;
    @Resource
    private SysRoleMenuMapper roleMenuMapper;
    @Resource
    private SysRoleDeptMapper roleDeptMapper;
    @Resource
    private SysUserRoleMapper userRoleMapper;


    /**
     * 新增角色
     *
     * @param role role
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertRole(SysRole role) {
        if (checkRoleNameNotUnique(role)) {
            Asserts.fail("新增角色'" + role.getRoleName() + "'失败，角色名称已存在");
        } else if (checkRoleKeyNotUnique(role)) {
            Asserts.fail("新增角色'" + role.getRoleName() + "'失败，角色权限已存在");
        }

        role.setCreateBy(sysUserService.getCurrentUser().getUserName());
        role.setCreateTime(new Date());
        roleMapper.insertRole(role);
        insertRoleMenu(role);
    }

    /**
     * 修改角色
     *
     * @param role role
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRole(SysRole role) {
        checkRoleAllowed(role);
        if (checkRoleNameNotUnique(role)) {
            Asserts.fail("修改角色'" + role.getRoleName() + "'失败，角色名称已存在");
        } else if (checkRoleKeyNotUnique(role)) {
            Asserts.fail("修改角色'" + role.getRoleName() + "'失败，角色权限已存在");
        }

        role.setUpdateBy(sysUserService.getCurrentUser().getUserName());
        role.setUpdateTime(new Date());
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        // 重新建立关系
        insertRoleMenu(role);
    }

    /**
     * 校验角色名称是否不唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleNameNotUnique(SysRole role) {
        SysRole info = roleMapper.checkRoleNameUnique(role.getRoleName());
        return EmptyUtil.isNotEmpty(info) && !info.getRoleId().equals(role.getRoleId());
    }

    /**
     * 校验角色权限是否不唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleKeyNotUnique(SysRole role) {
        SysRole info = roleMapper.checkRoleKeyUnique(role.getRoleKey());
        return EmptyUtil.isNotEmpty(info) && !info.getRoleId().equals(role.getRoleId());
    }

    /**
     * 校验角色是否允许操作
     *
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed(SysRole role) {
        if (EmptyUtil.isNotEmpty(role.getRoleId()) && role.isAdmin()) {
            Asserts.fail("不允许操作超级管理员角色");
        }
    }

    /**
     * 保存修改数据权限
     *
     * @param role role
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void authDataScope(SysRole role) {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        // 新增角色和部门信息（数据权限）
        insertRoleDept(role);
    }

    /**
     * 状态修改
     *
     * @param role role
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRoleStatus(SysRole role) {
        roleMapper.updateRole(role);
    }

    /**
     * 批量删除角色
     *
     * @param roleIds 角色 ids
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteRoleByIds(Long[] roleIds) {
        for (Long roleId : roleIds) {
            checkRoleAllowed(new SysRole(roleId));
            SysRole role = selectRoleById(roleId);
            if (countUserRoleByRoleId(roleId) > 0) {
                Asserts.fail(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDept(roleIds);
        roleMapper.deleteRoleByIds(roleIds);
    }

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    @Override
    public SysRole selectRoleById(Long roleId) {
        checkRoleDataScope(roleId);
        return roleMapper.selectRoleById(roleId);
    }

    /**
     * 通过角色ID查询角色使用数量
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId(Long roleId) {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    /**
     * 查询所有角色
     *
     * @return 所有角色
     */
    @Override
    public List<SysRole> selectRoleAll() {
        return SpringUtil.getAopProxy(this).selectRoleList(new SysRole());
    }

    /**
     * 批量选择授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要删除的用户数据ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertAuthUsers(Long roleId, Long[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<>();
        for (Long userId : userIds) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        userRoleMapper.batchUserRole(list);
    }

    /**
     * 批量取消授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要取消授权的用户数据ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAuthUsers(Long roleId, Long[] userIds) {
        userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }

    /**
     * 取消授权用户角色
     *
     * @param userRole userRole
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAuthUser(SysUserRole userRole) {
        userRoleMapper.deleteUserRoleInfo(userRole);
    }

    /**
     * 查询角色列表
     *
     * @param role 查询条件
     * @return 角色集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleList(SysRole role) {
        return roleMapper.selectRoleList(role);
    }

    /**
     * 查询用户拥有的所有角色
     *
     * @param userId 用户 id
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRolesByUserId(Long userId) {
        List<SysRole> userRoles = roleMapper.selectRolePermissionByUserId(userId);
        List<SysRole> roles = selectRoleAll();
        for (SysRole role : roles) {
            for (SysRole userRole : userRoles) {
                if (role.getRoleId().longValue() == userRole.getRoleId().longValue()) {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
    @Override
    public List<Integer> selectRoleListByUserId(Long userId) {
        return roleMapper.selectRoleListByUserId(userId);
    }

    /**
     * 根据用户ID查询角色权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(Long userId) {
        List<SysRole> perms = roleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (StringUtil.isNotNull(perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 新增角色菜单信息
     *
     * @param role 角色对象
     */
    @Transactional(rollbackFor = Exception.class)
    public void insertRoleMenu(SysRole role) {
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<>();
        for (Long menuId : role.getMenuIds()) {
            SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(role.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (list.size() > 0) {
            roleMenuMapper.batchRoleMenu(list);
        }

    }

    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    @Transactional(rollbackFor = Exception.class)
    public void insertRoleDept(SysRole role) {
        // 新增角色与部门（数据权限）管理
        List<SysRoleDept> list = new ArrayList<>();
        for (Long deptId : role.getDeptIds()) {
            SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0) {
            roleDeptMapper.batchRoleDept(list);
        }

    }

    /**
     * 校验角色是否有数据权限
     *
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope(Long roleId) {
        if (!sysUserService.getCurrentUser().isAdmin()) {
            SysRole role = new SysRole();
            role.setRoleId(roleId);
            List<SysRole> roles = SpringUtil.getAopProxy(this).selectRoleList(role);
            if (StringUtil.isEmpty(roles)) {
                Asserts.fail("没有权限访问角色数据！");
            }
        }
    }

    /**
     * 重置超级管理员权限
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refreshRole() {
        //查询所有菜单
        List<SysMenu> sysMenus = menuMapper.selectMenuList(new SysMenu());
        Long[] menuIds = new Long[sysMenus.size()];
        for (int i = 0; i < sysMenus.size(); i++) {
            menuIds[i] = sysMenus.get(i).getMenuId();
        }
        SysRole role = new SysRole();
        role.setRoleId(1L);
        role.setMenuIds(menuIds);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        // 重新建立关系
        insertRoleMenu(role);
    }
}
